@@ -2,5 +2,6 @@ from . import _version
 
 __version__ = _version.get_versions()["version"]
 
-from caf.space.inputs import ZoningTranslationInputs
+from caf.space.inputs import ZoningTranslationInputs, TransZoneSystemInfo, LowerZoneSystemInfo
 from caf.space.zone_translation import ZoneTranslation
+from caf.space.ui import SpaceUI
